@@ -341,15 +341,29 @@ EOEO
           if ($self->{runtime}->{options}->{hpacucli}) {
             #1 oder 0. pfad selber finden
             my $hpacucli = undef;
-            if (-e '/usr/sbin/hpacucli') {
+            if (-e '/usr/sbin/hpssacli') {
+              $hpacucli = '/usr/sbin/hpssacli';
+            } elsif (-e '/usr/local/sbin/hpssacli') {
+              $hpacucli = '/usr/local/sbin/hpssacli';
+            } elsif (-e '/usr/sbin/hpacucli') {
               $hpacucli = '/usr/sbin/hpacucli';
             } elsif (-e '/usr/local/sbin/hpacucli') {
               $hpacucli = '/usr/local/sbin/hpacucli';
+            } elsif (-e '/usr/sbin/hpssacli') {
+              $hpacucli = '/usr/sbin/hpssacli';
+            } elsif (-e '/usr/local/sbin/hpssacli') {
+              $hpacucli = '/usr/local/sbin/hpssacli';
             } else {
               $hpacucli = $hpasmcli;
               $hpacucli =~ s/^sudo\s*//;
               $hpacucli =~ s/hpasmcli/hpacucli/;
               $hpacucli = -e $hpacucli ? $hpacucli : undef;
+              if (! $hpacucli) {
+                $hpacucli = $hpasmcli;
+                $hpacucli =~ s/^sudo\s*//;
+                $hpacucli =~ s/hpasmcli/hpssacli/;
+                $hpacucli = -e $hpacucli ? $hpacucli : undef;
+              }
             }
             if ($hpacucli) {
               if ($< != 0) {
@@ -482,6 +496,8 @@ sub check_hpasm_client {
     } elsif (grep /must have a tty/, @output) {
       $self->add_message(CRITICAL,
           'sudo must be configured with requiretty=no (man sudo)');
+    } elsif (grep /ERROR: hpasmcli only runs on HPE Proliant Servers/, @output) {
+      $self->add_message(UNKNOWN, "hpasmcli detected incompatible hardware");
     } elsif (! grep /CLEAR/, @output) {
       $self->add_message(UNKNOWN,
           sprintf "insufficient rights to call %s", $hpasmcli);
