@@ -1,79 +1,156 @@
 #!/usr/bin/env python
-"""
-Nagios plugin for Amazon RDS monitoring.
+"""Nagios plugin for Amazon RDS monitoring.
 
-This program is part of Percona Monitoring Plugins
+This program is part of $PROJECT_NAME$
 License: GPL License (see COPYING)
 
 Author Roman Vynar
-Copyright 2014 Percona LLC and/or its affiliates
+Copyright 2014-2015 Percona LLC and/or its affiliates
 """
 
-import boto
 import datetime
 import optparse
 import pprint
 import sys
 
-def get_rds_info(indentifier=None):
-    """Function for fetching RDS details"""
-    rds = boto.connect_rds()
-    try:
-        if indentifier:
-            info = rds.get_all_dbinstances(indentifier)[0]
+import boto
+import boto.rds
+import boto.ec2.cloudwatch
+
+# Nagios status codes
+OK = 0
+WARNING = 1
+CRITICAL = 2
+UNKNOWN = 3
+
+
+class RDS(object):
+
+    """RDS connection class"""
+
+    def __init__(self, region, profile=None, identifier=None):
+        """Get RDS instance details"""
+        self.region = region
+        self.profile = profile
+        self.identifier = identifier
+
+        if self.region == 'all':
+            self.regions_list = [reg.name for reg in boto.rds.regions()]
         else:
-            info = rds.get_all_dbinstances()
-    except boto.exception.BotoServerError:
-        info = None
-    return info
+            self.regions_list = [self.region]
+
+        self.info = None
+        if self.identifier:
+            for reg in self.regions_list:
+                try:
+                    rds = boto.rds.connect_to_region(reg, profile_name=self.profile)
+                    self.info = rds.get_all_dbinstances(self.identifier)
+                except (boto.provider.ProfileNotFoundError, boto.exception.BotoServerError) as msg:
+                    debug(msg)
+                else:
+                    # Exit on the first region and identifier match
+                    self.region = reg
+                    break
+
+    def get_info(self):
+        """Get RDS instance info"""
+        if self.info:
+            return self.info[0]
+        else:
+            return None
+
+    def get_list(self):
+        """Get list of available instances by region(s)"""
+        result = dict()
+        for reg in self.regions_list:
+            try:
+                rds = boto.rds.connect_to_region(reg, profile_name=self.profile)
+                result[reg] = rds.get_all_dbinstances()
+            except (boto.provider.ProfileNotFoundError, boto.exception.BotoServerError) as msg:
+                debug(msg)
+
+        return result
+
+    def get_metric(self, metric, start_time, end_time, step):
+        """Get RDS metric from CloudWatch"""
+        cw_conn = boto.ec2.cloudwatch.connect_to_region(self.region, profile_name=self.profile)
+        result = cw_conn.get_metric_statistics(
+            step,
+            start_time,
+            end_time,
+            metric,
+            'AWS/RDS',
+            'Average',
+            dimensions={'DBInstanceIdentifier': [self.identifier]}
+        )
+        if result:
+            if len(result) > 1:
+                # Get the last point
+                result = sorted(result, key=lambda k: k['Timestamp'])
+                result.reverse()
+
+            result = float('%.2f' % result[0]['Average'])
+
+        return result
+
+
+def debug(val):
+    """Debugging output"""
+    global options
+    if options.debug:
+        print 'DEBUG: %s' % val
 
-def get_rds_stats(step, start_time, end_time, metric, indentifier):
-    """Function for fetching RDS statistics from CloudWatch"""
-    cw = boto.connect_cloudwatch()
-    result = cw.get_metric_statistics(step,
-        start_time,
-        end_time,
-        metric,
-        'AWS/RDS',
-        'Average',
-        dimensions={'DBInstanceIdentifier': [indentifier]})
-    if result:
-        if len(result) > 1:
-            # Get the last point
-            result = sorted(result, key=lambda k: k['Timestamp'])
-            result.reverse()
-        result = float('%.2f' % result[0]['Average'])
-    return result
 
 def main():
     """Main function"""
+    global options
 
-    # Nagios status codes
-    OK = 0
-    WARNING = 1
-    CRITICAL = 2
-    UNKNOWN = 3
-    short_status = {OK: 'OK',
-                    WARNING: 'WARN',
-                    CRITICAL: 'CRIT',
-                    UNKNOWN: 'UNK'}
+    short_status = {
+        OK: 'OK',
+        WARNING: 'WARN',
+        CRITICAL: 'CRIT',
+        UNKNOWN: 'UNK'
+    }
 
-    # DB instance classes as listed on http://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.DBInstanceClass.html
-    db_classes = {'db.t1.micro': 0.61,
-                  'db.m1.small': 1.7,
-                  'db.m1.medium': 3.75,
-                  'db.m1.large': 7.5,
-                  'db.m1.xlarge': 15,
-                  'db.m2.xlarge': 17.1,
-                  'db.m2.2xlarge': 34,
-                  'db.m2.4xlarge': 68,
-                  'db.cr1.8xlarge': 244}
+    # DB instance classes as listed on
+    # http://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.DBInstanceClass.html
+    db_classes = {
+        'db.t1.micro': 0.615,
+        'db.m1.small': 1.7,
+        'db.m1.medium': 3.75,
+        'db.m1.large': 7.5,
+        'db.m1.xlarge': 15,
+        'db.m4.large': 8,
+        'db.m4.xlarge': 16,
+        'db.m4.2xlarge': 32,
+        'db.m4.4xlarge': 64,
+        'db.m4.10xlarge': 160,
+        'db.r3.large': 15,
+        'db.r3.xlarge': 30.5,
+        'db.r3.2xlarge': 61,
+        'db.r3.4xlarge': 122,
+        'db.r3.8xlarge': 244,
+        'db.t2.micro': 1,
+        'db.t2.small': 2,
+        'db.t2.medium': 4,
+        'db.t2.large': 8,
+        'db.m3.medium': 3.75,
+        'db.m3.large': 7.5,
+        'db.m3.xlarge': 15,
+        'db.m3.2xlarge': 30,
+        'db.m2.xlarge': 17.1,
+        'db.m2.2xlarge': 34.2,
+        'db.m2.4xlarge': 68.4,
+        'db.cr1.8xlarge': 244,
+    }
 
-    # RDS metrics as listed on http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/rds-metricscollected.html
-    metrics = {'status': 'RDS availability',
-               'load': 'CPUUtilization',
-               'memory': 'FreeableMemory',
-               'storage': 'FreeStorageSpace'}
+    # RDS metrics http://docs.aws.amazon.com/AmazonCloudWatch/latest/DeveloperGuide/rds-metricscollected.html
+    metrics = {
+        'status': 'RDS availability',
+        'load': 'CPUUtilization',
+        'memory': 'FreeableMemory',
+        'storage': 'FreeStorageSpace'
+    }
 
     units = ('percent', 'GB')
 
@@ -81,34 +158,51 @@ def main():
     parser = optparse.OptionParser()
     parser.add_option('-l', '--list', help='list of all DB instances',
                       action='store_true', default=False, dest='db_list')
+    parser.add_option('-n', '--profile', default=None,
+                      help='AWS profile from ~/.boto or /etc/boto.cfg. Default: None, fallbacks to "[Credentials]".')
+    parser.add_option('-r', '--region', default='us-east-1',
+                      help='AWS region. Default: us-east-1. If set to "all", we try to detect the instance region '
+                           'across all of them, note this will be slower than if you specify the region explicitly.')
     parser.add_option('-i', '--ident', help='DB instance identifier')
     parser.add_option('-p', '--print', help='print status and other details for a given DB instance',
-                      action='store_true', default=False, dest='info')
+                      action='store_true', default=False, dest='printinfo')
     parser.add_option('-m', '--metric', help='metric to check: [%s]' % ', '.join(metrics.keys()))
     parser.add_option('-w', '--warn', help='warning threshold')
     parser.add_option('-c', '--crit', help='critical threshold')
-    parser.add_option('-u', '--unit', help='unit of thresholds for "storage" and "memory" metrics: [%s]. Default: percent' % ', '.join(units),
-                      default='percent')
-    options, args = parser.parse_args()
+    parser.add_option('-u', '--unit', help='unit of thresholds for "storage" and "memory" metrics: [%s]. '
+                      'Default: percent' % ', '.join(units), default='percent')
+    parser.add_option('-t', '--time', help='time period in minutes to query. Default: 5',
+                      type='int', default=5)
+    parser.add_option('-a', '--avg', help='time average in minutes to request. Default: 1',
+                      type='int', default=1)
+    parser.add_option('-d', '--debug', help='enable debug output',
+                      action='store_true', default=False)
+    options, _ = parser.parse_args()
+
+    if options.debug:
+        boto.set_stream_logger('boto')
+
+    rds = RDS(region=options.region, profile=options.profile, identifier=options.ident)
 
     # Check args
     if len(sys.argv) == 1:
         parser.print_help()
         sys.exit()
     elif options.db_list:
-        info = get_rds_info()
-        print 'List of all DB instances:'
+        info = rds.get_list()
+        print 'List of all DB instances in %s region(s):' % (options.region,)
         pprint.pprint(info)
         sys.exit()
     elif not options.ident:
         parser.print_help()
         parser.error('DB identifier is not set.')
-    elif options.info:
-        info = get_rds_info(options.ident)
+    elif options.printinfo:
+        info = rds.get_info()
         if info:
             pprint.pprint(vars(info))
         else:
-            print 'No DB instance "%s" found on your AWS account.' % options.ident
+            print 'No DB instance "%s" found on your AWS account and %s region(s).' % (options.ident, options.region)
+
         sys.exit()
     elif not options.metric or options.metric not in metrics.keys():
         parser.print_help()
@@ -119,15 +213,21 @@ def main():
     elif not options.crit and options.metric != 'status':
         parser.print_help()
         parser.error('Critical threshold is not set.')
+    elif options.avg <= 0 and options.metric != 'status':
+        parser.print_help()
+        parser.error('Average must be greater than zero.')
+    elif options.time <= 0 and options.metric != 'status':
+        parser.print_help()
+        parser.error('Time must be greater than zero.')
 
-    tm = datetime.datetime.utcnow()
+    now = datetime.datetime.utcnow()
     status = None
     note = ''
     perf_data = None
 
     # RDS Status
     if options.metric == 'status':
-        info = get_rds_info(options.ident)
+        info = rds.get_info()
         if not info:
             status = UNKNOWN
             note = 'Unable to get RDS instance'
@@ -137,6 +237,7 @@ def main():
                 version = info.EngineVersion
             except:
                 version = info.engine_version
+
             note = '%s %s. Status: %s' % (info.engine, version, info.status)
 
     # RDS Load Average
@@ -148,6 +249,7 @@ def main():
             fail = len(warns) + len(crits)
         except:
             fail = 0
+
         if fail != 6:
             parser.error('Warning and critical thresholds should be 3 comma separated numbers, e.g. 20,15,10')
 
@@ -159,16 +261,17 @@ def main():
             if i == 1:
                 # Some stats are delaying to update on CloudWatch.
                 # Let's pick a few points for 1-min load avg and get the last point.
-                n = 5
+                points = 5
             else:
-                n = i
-            load = get_rds_stats(i * 60, tm - datetime.timedelta(seconds=n * 60), tm,
-                                 metrics[options.metric], options.ident)
+                points = i
+
+            load = rds.get_metric(metrics[options.metric], now - datetime.timedelta(seconds=points * 60), now, i * 60)
             if not load:
                 status = UNKNOWN
                 note = 'Unable to get RDS statistics'
                 perf_data = None
                 break
+
             loads.append(str(load))
             perf_data.append('load%s=%s;%s;%s;0;100' % (i, load, warns[j], crits[j]))
 
@@ -181,11 +284,13 @@ def main():
                     fail = True
                 elif load >= warns[j]:
                     status = WARNING
+
             j = j + 1
 
         if status != UNKNOWN:
             if status is None:
                 status = OK
+
             note = 'Load average: %s%%' % '%, '.join(loads)
             perf_data = ' '.join(perf_data)
 
@@ -198,15 +303,17 @@ def main():
             crit = float(options.crit)
         except:
             parser.error('Warning and critical thresholds should be integers.')
+
         if crit > warn:
             parser.error('Parameter inconsistency: critical threshold is greater than warning.')
+
         if options.unit not in units:
             parser.print_help()
             parser.error('Unit is not valid.')
 
-        info = get_rds_info(options.ident)
-        free = get_rds_stats(60, tm - datetime.timedelta(seconds=60), tm,
-                             metrics[options.metric], options.ident)
+        info = rds.get_info()
+        free = rds.get_metric(metrics[options.metric], now - datetime.timedelta(seconds=options.time * 60),
+                              now, options.avg * 60)
         if not info or not free:
             status = UNKNOWN
             note = 'Unable to get RDS details and statistics'
@@ -219,6 +326,7 @@ def main():
                 except:
                     print 'Unknown DB instance class "%s"' % info.instance_class
                     sys.exit(CRITICAL)
+
             free = '%.2f' % (free / 1024 ** 3)
             free_pct = '%.2f' % (float(free) / storage * 100)
             if options.unit == 'percent':
@@ -236,6 +344,7 @@ def main():
 
             if status is None:
                 status = OK
+
             note = 'Free %s: %s GB (%.0f%%) of %s GB' % (options.metric, free, float(free_pct), storage)
             perf_data = 'free_%s=%s;%s;%s;0;%s' % (options.metric, val, warn, crit, val_max)
 
@@ -244,8 +353,10 @@ def main():
         print '%s %s | %s' % (short_status[status], note, perf_data)
     else:
         print '%s %s' % (short_status[status], note)
+
     sys.exit(status)
 
+
 if __name__ == '__main__':
     main()
 
@@ -266,6 +377,13 @@ pmp-check-aws-rds.py - Check Amazon RDS metrics.
   Options:
     -h, --help            show this help message and exit
     -l, --list            list of all DB instances
+    -n PROFILE, --profile-name=PROFILE
+                          AWS profile from ~/.boto or /etc/boto.cfg. Default:
+                          None, fallbacks to "[Credentials]".
+    -r REGION, --region=REGION
+                          AWS region. Default: us-east-1. If set to "all", we
+                          try to detect the instance region across all of them,
+                          note this will be slower than you specify the region.
     -i IDENT, --ident=IDENT
                           DB instance identifier
     -p, --print           print status and other details for a given DB instance
@@ -275,8 +393,11 @@ pmp-check-aws-rds.py - Check Amazon RDS metrics.
     -c CRIT, --crit=CRIT  critical threshold
     -u UNIT, --unit=UNIT  unit of thresholds for "storage" and "memory" metrics:
                           [percent, GB]. Default: percent
+    -t TIME, --time=TIME  time period in minutes to query. Default: 5
+    -a AVG, --avg=AVG     time average in minutes to request. Default: 1
+    -d, --debug           enable debug output
 
-=head1 REQUIREMENTS 
+=head1 REQUIREMENTS
 
 This plugin is written on Python and utilizes the module C<boto> (Python interface
 to Amazon Web Services) to get various RDS metrics from CloudWatch and compare
@@ -313,78 +434,96 @@ The plugin provides 4 checks and some options to list and print RDS details:
 
 To get the list of all RDS instances under AWS account:
 
-  # ./aws-rds-nagios-check.py -l
+  # ./pmp-check-aws-rds.py -l
 
-To get the detailed status of RDS instance identified as C<blackbox>: 
+To get the detailed status of RDS instance identified as C<blackbox>:
 
-  # ./aws-rds-nagios-check.py -i blackbox -p
+  # ./pmp-check-aws-rds.py -i blackbox -p
 
 Nagios check for the overall status. Useful if you want to set the rest
 of the checks dependent from this one:
 
-  # ./aws-rds-nagios-check.py -i blackbox -m status
+  # ./pmp-check-aws-rds.py -i blackbox -m status
   OK mysql 5.1.63. Status: available
 
 Nagios check for CPU utilization, specify thresholds as percentage of
-1-min., 5-min., 15-min. average accordingly: 
+1-min., 5-min., 15-min. average accordingly:
 
-  # ./aws-rds-nagios-check.py -i blackbox -m load -w 90,85,80 -c 98,95,90
+  # ./pmp-check-aws-rds.py -i blackbox -m load -w 90,85,80 -c 98,95,90
   OK Load average: 18.36%, 18.51%, 15.95% | load1=18.36;90.0;98.0;0;100 load5=18.51;85.0;95.0;0;100 load15=15.95;80.0;90.0;0;100
 
 Nagios check for the free memory, specify thresholds as percentage:
 
-  # ./aws-rds-nagios-check.py -i blackbox -m memory -w 5 -c 2
+  # ./pmp-check-aws-rds.py -i blackbox -m memory -w 5 -c 2
   OK Free memory: 5.90 GB (9%) of 68 GB | free_memory=8.68;5.0;2.0;0;100
-  # ./aws-rds-nagios-check.py -i blackbox -m memory -u GB -w 4 -c 2
+  # ./pmp-check-aws-rds.py -i blackbox -m memory -u GB -w 4 -c 2
   OK Free memory: 5.90 GB (9%) of 68 GB | free_memory=5.9;4.0;2.0;0;68
 
-Nagios check for the free storage space, specify thresholds as percentage or GB: 
+Nagios check for the free storage space, specify thresholds as percentage or GB:
 
-  # ./aws-rds-nagios-check.py -i blackbox -m storage -w 10 -c 5
+  # ./pmp-check-aws-rds.py -i blackbox -m storage -w 10 -c 5
   OK Free storage: 162.55 GB (33%) of 500.0 GB | free_storage=32.51;10.0;5.0;0;100
-  # ./aws-rds-nagios-check.py -i blackbox -m storage -u GB -w 10 -c 5
+  # ./pmp-check-aws-rds.py -i blackbox -m storage -u GB -w 10 -c 5
   OK Free storage: 162.55 GB (33%) of 500.0 GB | free_storage=162.55;10.0;5.0;0;500.0
 
+By default, the region is set to ``us-east-1``. You can re-define it globally in boto config or
+specify with -r option. The following command will list all instances across all regions under your AWS account:
+
+  # ./pmp-check-aws-rds.py -r all -l
+
+The following command will show the status for the first instance identified as ``blackbox`` in all regions:
+
+  # ./pmp-check-aws-rds.py -r all -i blackbox -p
+
+Remember, scanning regions are slower operation than specifying it explicitly.
+
 =head1 CONFIGURATION
 
 Here is the excerpt of potential Nagios config:
 
+  define host{
+        use                             mysql-host
+        host_name                       blackbox
+        alias                           blackbox
+        address                         blackbox.abcdefgh.us-east-1.rds.amazonaws.com
+        }
+
   define servicedependency{
-        hostgroup_name                  mysql-servers
+        host_name                       blackbox
         service_description             RDS Status
-        dependent_service_description   RDS Load Average, RDS Free Storage, RDS Free Memory 
+        dependent_service_description   RDS Load Average, RDS Free Storage, RDS Free Memory
         execution_failure_criteria      w,c,u,p
         notification_failure_criteria   w,c,u,p
         }
-  
+
   define service{
         use                             active-service
-        hostgroup_name                  mysql-servers
+        host_name                       blackbox
         service_description             RDS Status
         check_command                   check_rds!status!0!0
         }
-       
+
   define service{
         use                             active-service
-        hostgroup_name                  mysql-servers
+        host_name                       blackbox
         service_description             RDS Load Average
         check_command                   check_rds!load!90,85,80!98,95,90
         }
-  
+
   define service{
         use                             active-service
-        hostgroup_name                  mysql-servers
+        host_name                       blackbox
         service_description             RDS Free Storage
         check_command                   check_rds!storage!10!5
         }
-  
+
   define service{
         use                             active-service
-        hostgroup_name                  mysql-servers
+        host_name                       blackbox
         service_description             RDS Free Memory
         check_command                   check_rds!memory!5!2
         }
-  
+
   define command{
         command_name    check_rds
         command_line    $USER1$/pmp-check-aws-rds.py -i $HOSTALIAS$ -m $ARG1$ -w $ARG2$ -c $ARG3$
@@ -407,7 +546,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 
 =head1 VERSION
 
-Percona Monitoring Plugins pmp-check-aws-rds.py 1.1.4
+$PROJECT_NAME$ pmp-check-aws-rds.py $VERSION$
 
 =cut
 
